@@ -1,29 +1,78 @@
 from ursina import *
+from panda3d.core import Texture
+from . import common
+import cv2
 
 class Entity4t(Entity):
     def __init__(self, *args, **kwargs):
         # handle default model and texture
+        #custom model
+        
+        
+        #if not 'model' in kwargs or kwargs['model'] == 'cube':
         if not 'model' in kwargs:
+            # add_entity
+            pass
+        elif kwargs['model'] == 'cube':
+            # default cube model 
             kwargs['model'] = 'cube'
             kwargs['texture'] = 'white_cube.png'
+        elif kwargs['model'] == 'sphere':
+            kwargs['texture'] = load_texture('grid_white',common.texture4t_folder)
 
-        if not 'collider' in kwargs:
-            kwargs['collider'] = 'box'
+        elif kwargs['model'] == 'quad':
+            kwargs['texture'] = 'white_cube.png'
+            #kwargs['double_sided'] = True
+            pass
+
+        elif kwargs['model'] == 'cubic_six_faces':
+            # cubic with 6 face texture
+            kwargs['model'] = load_model('cubic6', common.model4t_folder)
+            kwargs['texture'] = load_texture('cubic6_colors',common.texture4t_folder)
+            #print(self._texture)
+            #print('here')
+        elif kwargs['model'] == 'sphere_inward':
+            kwargs['model'] = load_model('sphere_inward', common.model4t_folder)
+            kwargs['texture'] = load_texture('abc_grid',common.texture4t_folder)
+        elif kwargs['model'] == 'cube_line':
+            kwargs['model'] = load_model('cube_line', common.model4t_folder)  
+        elif kwargs['model'] == 'tetrahedron_line':
+            kwargs['model'] = load_model('tetrahedron_line', common.model4t_folder)       
+        else:
+            pass
+
+        #if not 'collider' in kwargs:
+        #    kwargs['collider'] = 'box'
         
+
+
+
         super().__init__(*args, **kwargs)
 
     def 位置動畫(self, 座標, 延遲=0, 持續=1):
         #座標 = Vec3(座標差[0], 座標差[1], 座標差[2],) + self.position
-        self.animate_position(座標, delay=延遲 ,duration=持續)
+        self.animate_position(座標, delay=延遲 ,duration=持續, curve=curve.linear)
 
     def 旋轉動畫(self, 角度, 延遲=0, 持續=1):    
-        self.animate_rotation(角度, delay=延遲 ,duration=持續)
+        self.animate_rotation(角度, delay=延遲 ,duration=持續, curve=curve.linear)
 
     def 縮放動畫(self, 比例, 延遲=0, 持續=1):    
-        self.animate_scale(比例, delay=延遲 ,duration=持續)
+        self.animate_scale(比例, delay=延遲 ,duration=持續, curve=curve.linear)
 
     def 顏色動畫(self, 顏色, 延遲=0, 持續=1):    
-        self.animate_color(顏色, delay=延遲 ,duration=持續)
+        self.animate_color(顏色, delay=延遲 ,duration=持續, curve=curve.linear)
+
+    def 淡出(self, 持續=1):
+        self.fade_out(duration=持續)
+
+    def 淡入(self, 持續=1):
+        self.fade_in(duration=持續, curve=curve.linear)
+
+    def 閃爍(self, 持續=0.5):
+        self.blink(duration=持續, curve=curve.linear)
+
+    def 晃動(self):
+        self.shake()
 
     @property
     def 模型(self):
@@ -37,14 +86,44 @@ class Entity4t(Entity):
         #self.setTextureOff(True)
 
     @property
-    def 紋理(self):
+    def 材質貼圖(self):
         self.setTextureOff(True)
         return self.texture 
 
     @模型.setter
-    def 紋理(self, value):
+    def 材質貼圖(self, value):
         self.texture = value
 
+    @property
+    def 多維陣列貼圖(self):
+        self.setTextureOff(True)
+        return self.texture 
+
+    @模型.setter
+    def 多維陣列貼圖(self, buf):
+        buf = cv2.flip(buf, -1)
+        width = buf.shape[1]
+        height = buf.shape[0]
+        #print(width, height)
+
+        
+        common.ndarray_texure.setup2dTexture(width,height,
+                Texture.T_unsigned_byte,Texture.F_rgb8)
+        common.ndarray_texure.setRamImage(buf)
+        
+        # remove texture
+        self.texture = None
+
+        self.setTexture(common.ndarray_texure, True)
+        #print('tex ',common.ndarray_texure)
+        #self.texture = tex
+
+    @property
+    def 動畫清單(self):
+        return self.animations 
+
+
+
     @property
     def 上層物件(self):
         return self.parent
@@ -275,5 +354,20 @@ class Entity4t(Entity):
     def 全域縮放z(self, value):
         self.world_scale_z = value 
 
+    @property
+    def 兩面貼圖(self):
+        return self.double_sided 
+
+    @兩面貼圖.setter
+    def 兩面貼圖(self, value):
+        self.double_sided = value
+
+    @property
+    def 著色器(self):
+        return self.shader 
+
+    @著色器.setter
+    def 著色器(self, value):
+        self.shader = value
 
     
\ No newline at end of file
